# $Id: Subscription.pm,v 1.5 2004/05/15 12:21:16 rich Exp $

package Email::List::Subscription;

use strict;
use Email::List::Subscription::Welcome;
use Email::List::Subscription::Status;
use Email::List::Subscription::Goodbye;
use HTTP::Date;

our $SUBSCRIPTION = 1;
our $UNSUBSCRIPTION = 2;

sub new
{
    my $class = shift(@_);
    my $listname = shift(@_);
    my $subscriber = shift(@_);
    my $type = shift(@_);
    my $date = shift(@_);

    return bless({ listname => $listname,
		   subscriber => $subscriber,
		   type => $type,
		   date => $date },
		 $class);
}

sub _add
{
    my $self = shift(@_);
    my $data = shift(@_);
    my $field = shift(@_);

    $self->{$field} = $data;
}

sub _delete
{
    my $self = shift(@_);
    my $field = shift(@_);

    undef $self->{$field};
}

sub add_url
{
    _add(@_, 'url');
}

sub delete_url
{
    _delete(@_, 'url');
}

sub add_password
{
    _add(@_, 'password');
}

sub delete_password
{
    _add(@_, 'password');
}

sub match
{
    my $class = shift(@_);
    my $message = shift(@_);
    my @subscriptions;

    push (@subscriptions, Email::List::Subscription::Welcome::match($message));
    push (@subscriptions, Email::List::Subscription::Status::match($message));
    push (@subscriptions, Email::List::Subscription::Goodbye::match($message));

    return @subscriptions;
}

sub cmp_date ($$)
{
    my ($a, $b) = @_;
    my $a_time = str2time($a->{date}, 'GMT');
    my $b_time = str2time($b->{date}, 'GMT');

    if (!defined($a_time) || !defined($b_time))
    {
	# Bogus date format. So assume that $b was parsed after $a
	# and that mailboxes store mails in increasing time order.
	#
	# TODO: This is probably broken for Maildirs.
	$a_time = 0;
	$b_time = 1;
    }

    return $a_time - $b_time;
}

1;
